#!/usr/bin/python

import sys
import os
import commands
import string
import tempfile

################################# CONFIGURATION ########################
from socket import gethostname

if gethostname() in ["gwydion.winters-ca.net", "hebe.cs.ucr.edu"]:
    JAVA_PATH = "/usr/bin/java"
    MULTIVALENT_PATH = "/home/titus/bin/Multivalent.jar"
else:
    JAVA_PATH = "/usr/local/bin/java"
    MULTIVALENT_PATH = "/usr/local/pkgs/Multivalent/Multivalent.jar"

########################### Known file format extensions ###############
BMP_TYPES = ["bmp", "gif", "jpg", "jpeg", "pcx", "pbm", "ppm", "png", "tga",
             "tif", "tiff"]
TXT_TYPES = ["txt", "c", "cc", "cpp", "h", "hh", "py", "pl", "html"]

DOC_TYPES = ["doc", "rtf", "sxw", "abw"]

PS_TYPES = ["ps"]

# Brief usage message
def usage():    
    print "Usage:"
    print sys.argv[0], "outputPDF file1 [files]"
    print "Convert file1 ... into PDF, and concatenate them in the order"
    print "given into outputPDF."
    sys.exit(1)

# Given a bitmap that needs converting, do the conversion and return
# the filename
def bmpConverter(filename, tempDir):
    # Get the intermediate step and return filename
    tempPs = tempfile.mkstemp(dir=tempDir, suffix=".ps")[1]
    tempPdf = tempfile.mkstemp(dir=tempDir, suffix=".pdf")[1]
    status, output = commands.getstatusoutput("convert %s %s" % \
                                              (filename, tempPs))
    # Did that break?
    if status:
        print >>sys.stderr, output
        raise RuntimeError

    # Do the final convert to pdf
    status, output = commands.getstatusoutput("ps2pdf %s %s" % \
                                              (tempPs, tempPdf))
    if status:
        print >>sys.stderr, output
        raise RuntimeError

    # Everything worked, return the new filename
    return tempPdf

# Given a text (hopefully) filename and a directory to store temp
# results in, do the conversion to PDF and return the filename
def txtConverter(filename, tempDir):
    # The intermediate step
    tempPs = tempfile.mkstemp(dir=tempDir, suffix=".ps")[1]

    # The return value
    tempPdf = tempfile.mkstemp(dir=tempDir, suffix=".pdf")[1]

    # a2ps on 8.5x11" paper, Portrait mode, single-page-per-page
    cmdLine = "a2ps -M letter -R --columns=1 --rows=1 %s -o %s" % \
              (filename, tempPs)
    status, output = commands.getstatusoutput(cmdLine)

    # if that didn't work, break
    if status:
        print >>sys.stderr, output
        raise RuntimeError

    # final convert step
    status, output = commands.getstatusoutput("ps2pdf %s %s" % \
                                              (tempPs, tempPdf))
    if status:
        print >>sys.stderr, output
        raise RuntimeError

    return tempPdf

# Given a .doc filename and a directory to store temp
# results in, do the conversion to PDF and return the filename
def docConverter(filename, tempDir):
    # The intermediate step
    tempPs = tempfile.mkstemp(dir=tempDir, suffix=".ps")[1]

    # The return value
    tempPdf = tempfile.mkstemp(dir=tempDir, suffix=".pdf")[1]

    # a2ps on 8.5x11" paper, Portrait mode, single-page-per-page
    cmdLine = "abiword --print=%s %s" % \
              (tempPs, filename)
    status, output = commands.getstatusoutput(cmdLine)

    # if that didn't work, break
    if status:
        print >>sys.stderr, output
        raise RuntimeError

    # final convert step
    status, output = commands.getstatusoutput("ps2pdf %s %s" % \
                                              (tempPs, tempPdf))
    if status:
        print >>sys.stderr, output
        raise RuntimeError

    return tempPdf

# Given a .doc filename and a directory to store temp
# results in, do the conversion to PDF and return the filename
def psConverter(filename, tempDir):
    # The return value
    tempPdf = tempfile.mkstemp(dir=tempDir, suffix=".pdf")[1]

    # final convert step
    status, output = commands.getstatusoutput("ps2pdf %s %s" % \
                                              (filename, tempPdf))
    if status:
        print >>sys.stderr, output
        raise RuntimeError

    return tempPdf


# In case there are symlinks involved here
JAVA_PATH = os.path.realpath(JAVA_PATH)
MULTIVALENT_PATH = os.path.realpath(MULTIVALENT_PATH)

# Make sure we can find everything that would cause this to fail.
try:
    assert(os.access(JAVA_PATH, os.X_OK|os.R_OK))
except:
    sys.exit("Unable to access " + JAVA_PATH)

try:
    assert(os.access(MULTIVALENT_PATH, os.R_OK))
except:
    sys.exit("Unable to read " + MULTIVALENT_PATH)

try:
    assert(commands.getstatusoutput("which convert")[0] == 0)
except:
    sys.exit("Unable to find convert for bitmap files.")

try:
    assert(commands.getstatusoutput("which a2ps")[0] == 0)
except:
    sys.exit("Unable to find a2ps.")

try:
    assert(commands.getstatusoutput("which ps2pdf")[0] == 0)
except:
    sys.exit("Unable to find ps2pdf.")
    
try:
    assert(commands.getstatusoutput("which abiword")[0] == 0)
except:
    sys.exit("Unable to find abiword.")
    
    
# This is how we invoke Multivalent.
MULTIVALENT = "%s -classpath %s " % (JAVA_PATH, MULTIVALENT_PATH)

######################### Actual Program #######################
if len(sys.argv) < 3:
    usage()

# The directory that we will hold all of our temporary results in
tempdir = tempfile.mkdtemp()

# Make sure that we are putting stuff into a PDF 
assert(string.lower(sys.argv[1][-4:]) == ".pdf")

# This list of PDFs that we are going to 
pdfList = []
for file in sys.argv[2:]:
    # Figure out the extension
    ext = string.lower(file.split(".")[-1])

    # Do the right conversion
    if ext in BMP_TYPES:
        pdf = bmpConverter(file, tempdir)
    elif ext in TXT_TYPES:
        pdf = txtConverter(file, tempdir)
    elif ext in DOC_TYPES:
        pdf = docConverter(file, tempdir)
    elif ext in PS_TYPES:
        pdf = psConverter(file, tempdir)
    elif ext == "pdf":
        pdf = file
    else:
        # Warning: Uhh . . . .
        print "Warning: I don't know about type", ext
        print "Guessing text."
        pdf = txtConverter(file, tempdir)

    # Got a new one
    pdfList.append(pdf)

# Do the PDF concatenation
cmdLine = MULTIVALENT + "tool.pdf.Merge -append " + string.join(pdfList, " ")
status, output = commands.getstatusoutput(cmdLine)

# If that failed, fail
if status:
    print >>sys.stderr, output
    sys.exit(status)

# That merged everything into pdfList[0], now move to
# sys.argv[1]
commands.getstatusoutput("cp %s %s" % (pdfList[0], sys.argv[1]))

# And clean up our temp directory
commands.getstatusoutput("rm -rf " + tempdir)
