#!/usr/bin/python

import sys
import commands
import getopt
import popen2
import os
import select
import signal
import string
import time
import thread

def usage():
    print "Usage: bulktest.py OUTPUT EXECUTABLE points"
    print "Prints the number of tests that the given executable passed."
    print "The success of a test is determined by checking if the output"
    print "matches the output given by the file specified by the output"
    print "flag.  Each test is separated by a line of dashes ('-')."
    print
    print "--output=FILE \tWhat output the executable must produce "
    print "              \t(no input).  Lines consisting of only -'s"
    print "              \tare interpreted as breaks between tests."
    print "--ignore-extra\tIf this flag is provided, then extra LINES of"
    print "              \toutput from the executable will be ignored."
    print "--forgive-segfault\tGrant earned partial credit even if program"
    print "                  \tterminates abnormally."
    print "--timeout=secs\tHow many seconds before giving up."
    
    sys.exit(2)

def defaults():
    print "Bulk Test"
    print "one"
    print "*graderExe"
    usage()

def countTests(data):
    if type(data) == type(""):
        data = data.split("\n")
        
    ret = 0
    for line in data:
        if len(line) > 1 and line.count("-") >= len(line) - 1:
            ret += 1
    return ret

def splitTests(data):
    ret = []
    cur = []
    if type(data) == type(""):
        data = data.split("\n")
    for line in data:
        if len(line) > 1 and line.count("-") >= len(line) - 1:
            ret.append(cur)
            cur = []
        else:
            cur.append(line)
    return ret

try:
    opts, args = getopt.getopt(sys.argv[1:], "", ["help", "defaults",
                                                  "output=",
                                                  "timeout=",
                                                  "maxpoints=",
                                                  "ignore-extra",
                                                  "forgive-segfault"])

    
except getopt.GetoptError:
    # print help information and exit:
    usage()

output = None
timeout = 3
ignore = 0
forgive = False
for o, a in opts:
    if o == "--help":
        usage()
    if o == "--defaults":
        defaults()

    if o == "--ignore-extra":
        ignore = 1

    if o == "--forgive-segfault":
        forgive = True

    try:        
        if o == "--output":
            output = a

        if o == "--timeout":
            timeout = float(a)

    except:
        usage()

if not output:
    usage()

output = os.path.expanduser(output)

try:
    maxpoints = float(sys.argv.pop())
except:
    print "Bad point value."
    sys.exit(1)

try:
    file = open(sys.argv[-1])
except:
    print "Unable to open", sys.argv[-1], "for reading"
    sys.exit(1)

raw = open(output).readlines()
data = ""
for i in raw:
    data += i

os.chdir(os.path.dirname(sys.argv[-1]))
#print "Changing to", os.path.dirname(sys.argv[-1])

sys.argv[-1] = "./" + os.path.basename(sys.argv[-1])
#print "Running", sys.argv[-1], "in", os.getcwd()
childProc = popen2.Popen3(sys.argv[-1], True)

startTime = time.time()
output = ""
while 1:
    cur = time.time()
    timeLeft = timeout - (cur - startTime)

    if timeLeft < 0:
        break
    
    (r, w, o) = select.select([childProc.fromchild], [], [], timeLeft)

    if not r:
        timeLeft = 0
        break

    if childProc.fromchild.closed:
        break

    curline = childProc.fromchild.readline()
    if not curline:
        break

    output += curline

if timeLeft <= 0:
    print 0
    print "Timeout after", timeout, "seconds, aborting."    
    raw = commands.getoutput('ps -ef | grep "\\W' + str(childProc.pid) + \
                             '\\W"')
    lines = raw.split("\n")
    for line in lines:
        pid = int(line.split()[1])
        if pid != childProc.pid:
            os.kill(pid, signal.SIGKILL)
            
    os.kill(childProc.pid, signal.SIGINT)
    status = childProc.wait()
    if status:
        if not forgive:
            if status == 35584:
                print "Segmentation Fault"
            elif status == 34304:
                print "Aborted"
            else:
                print "Unexpected termination, aborting with exit status", \
                      status
    sys.exit(0)
            
status = childProc.wait()
if status and not forgive:
    print 0
    if status == 35584:
        print "Segmentation Fault"
    elif status == 34304:
        print "Aborted"
    else:
        print "Unexpected termination, aborting with exit status", status

    sys.exit(0)

# Now we need to go through the output and the sample output (data) and
# find where the tests are and such.

if countTests(output) > countTests(data):
    print "Different number of test cases, aborting"
    sys.exit(1)

testResults = splitTests(output)
testKeys = splitTests(data)

testResults += [None] * (len(testKeys) - len(testResults))

correct = 0
output = ""
for res, sample in zip(testResults, testKeys):
    if res == sample:
        correct += 1
    elif res == None:
        output += "No output\n"+ "-"*40 +"\n"    
        continue
    else:        
        if not ignore:
            output += string.join(res, "\n") + "\n"+ "-"*40 +"\n"    
            continue
        
        # If the ignore-extra flag was provided, then we
        # can ignore extra lines provided by the student's executable,
        # so long as all the lines in the sample output are actually
        # present.
        good = 1
        for i in sample:
            if res and i not in res:
                good = 0
                break
        if good:
            correct += 1
        else:
            output += string.join(res, "\n") + "\n"+ "-"*40 +"\n"    

if maxpoints != None:
    possible = len(testKeys)
    print round((1.0 * correct / possible) * maxpoints, 1)
else:
    print correct
if output:
    print "Incorrect test results"
    print output
sys.exit(0)

        
        

