#!/usr/bin/python

import commands
import difflib
import getopt
import os
import os.path
import popen2
import select
import signal
import string
import sys
import tempfile
import time

def usage():
    print "Usage: editdist.py [OPTION]... EXECUTABLE points"
    print
    print "--infile=FILE \t\tWhat input should be given to the executable."
    print "--outfile=FILE\t\tWhat output the executable must produce on "
    print "              \t\tthat input."
    print "--percentage=VAL\t\tWhat 'percentage' the output must match to be"
    print "                \t\tbe regarded as valid"
    print "--timeout=secs\t\tHow many seconds before giving up."
    print "--as-argument \t\tGive the program the argument 'infile' rather"
    print "              \t\tthan sending the contents of infile to standard"
    print "              \t\tinput."
    
    sys.exit(2)

def defaults():
    print "Edit Distance Test"
    print "one"
    print "*graderExe"
    print "{'timeout': 3, 'percentage': 60}"
    usage()


try:
    opts, args = getopt.getopt(sys.argv[1:], "", ["help", "defaults",
                                                  "infile=",
                                                  "outfile=",
                                                  "percentage=",
                                                  "timeout=",
                                                  "as-argument"])

    
except getopt.GetoptError:
    # print help information and exit:
    usage()
    sys.exit(0)

infile = None
outfile = None
distance = None
percentage = 60
timeout = 3
asArg = False

for o, a in opts:
    if o == "--help":
        usage()
    if o == "--defaults":
        defaults()

    try:        
        if o == "--infile":
            infile = a

        if o == "--outfile":
            outfile = a

        if o == "--percentage":
            percentage = int(a)

        if o == "--timeout":
            timeout = float(a)

        if o == "--as-argument":
            asArg = True

    except:
        usage()

if not infile or not outfile:
    usage()

infile = os.path.expanduser(infile)
outfile = os.path.expanduser(outfile)

try:
    points = float(sys.argv.pop())
except:
    print "Invalid point value"
    sys.exit(2)

try:
    file = open(sys.argv[-1])
except:
    print "Unable to open", sys.argv[-1], "for reading"
    usage()
   
raw = open(outfile).readlines()
data = ""
for i in raw:
    data += i
#data = data[:-1]

if asArg:
    childProc = popen2.Popen3(sys.argv[-1] + " " + infile)
else:
    childProc = popen2.Popen3(sys.argv[-1] + "<" + infile)

startTime = time.time()
output = ""
while 1:
    cur = time.time()
    timeLeft = timeout - (cur - startTime)

    if timeLeft < 0:
        break
    
    (r, w, o) = select.select([childProc.fromchild], [], [], timeLeft)

    if not r:
        timeLeft = 0
        break

    if childProc.fromchild.closed:
        break

    curline = childProc.fromchild.readline()
    if not curline:
        break

    output += curline

if timeLeft <= 0:
    print 0
    print "Timeout after", timeout, "seconds, aborting."    
    raw = commands.getoutput('ps -ef | grep "\\W' + str(childProc.pid) + \
                             '\\W"')
    lines = raw.split("\n")
    for line in lines:
        pid = int(line.split()[1])
        if pid != childProc.pid:
            os.kill(pid, signal.SIGKILL)
            
    os.kill(childProc.pid, signal.SIGINT)
    status = childProc.wait()
    if status:        
        if status == 35584:
            print "Segmentation Fault"
        elif status == 34304:
            print "Aborted"
        else:
            print "Unexpected termination, aborting with exit status", status
    
    sys.exit(0)

(file, filename) = tempfile.mkstemp()
os.write(file, output + "\n")
os.close(file)

(status, output) = commands.getstatusoutput("diff  -ibB " + \
                                            outfile + " " + filename)
if not status or len(output) * 20 < len(data):
    print points
    print "No significant output difference found."
    sys.exit(0)
else:
    print 0
    print output
    sys.exit(0)

s = difflib.SequenceMatcher(None, data, output)
if (s.quick_ratio() * 100 < percentage):
    print 0
    print "Percentage match:", s.quick_ratio() * 100
    bad = 1
elif (s.ratio() * 100 > percentage):
    print points
    print "Percentage match:", s.quick_ratio() * 100
    bad = 0
else:
    print 0
    print "Percentage match:", s.ratio() * 100
    bad = 1

if bad:
    print output

sys.exit(0)
