#!/usr/bin/python

import sys
import getopt

def usage():
    print "Usage: linewraps.py [OPTION]... FILE POINTS"
    print "If there are linewraps in FILE, print 0 and a message",
    print "to stdout and exit 0."
    print "If not, print POINTS and a message to stdout and exit 0."
    print "If there is an error, exit 1."
    print
    print "--line-len=LEN\t\tHow many characters counts as a line wrap?"
    print "              \t\tDefault=80"
    print "--tab-stops=SIZE\t\tMax spacing for a \\t character."
    print "              \t\tDefault=8"
    print "--tab-const=SIZE\t\tInterpret tabs as a constant size, rather"
    print "              \t\tthan rounding up to the next tab stop."
    sys.exit(2)

def defaults():
    # Name
    print "Fail on Line Wraps"

    # Fail when?
    print "one"

    # Default filemask?
    print "*.*"

    # Configuration
    print "{'line len': 80, 'tab stops': 8}"

    # Then print the usage statement
    usage()

try:
    opts, args = getopt.getopt(sys.argv[1:], "", ["help",
                                                  "defaults",
                                                  "line-len=",
                                                  "tab-stops=",
                                                  "tab-const="])

    
except getopt.GetoptError:
    # print help information and exit:
    usage()
    sys.exit(0)

linelen = 80
tabstops = 8
tabconst = 0

for o, a in opts:
    if o == "--help":
        usage()
    if o == "--defaults":
        defaults()

    try:        
        if o == "--line-len":
            linelen = int(a)

        if o == "--tab-stops":
            tabstops = int(a)

        if o == "--tab-const":
            tabconst = int(a)

    except:
        usage()

try:
    filename = sys.argv[-2]
except:
    print "Unable to open", sys.argv[-2], "for reading"
    sys.exit(1)

try:
    points = float(sys.argv[-1])
except:
    print "Invalid point specification", sys.argv[-1]
    sys.exit(1)
    
message = ""
lineNum = 1
for line in open(filename):
    size = len(line)
    if "\t" in line:
        if tabconst:
            size += tabstops * line.count("\t")
        else:
            for i in range(len(line)):
                extra = 0
                if line[i] == "\t":
                    extra += tabstops - (i + extra) % tabstops
                size += extra

    if (size > linelen):
        message += "Linewrap on line %d, %d character(s) too long\n" %\
                   (lineNum, size-linelen)

    lineNum += 1

if message:
    print 0
    print message
else:
    print points
    print "No linewraps found."

sys.exit(0)


