#!/usr/bin/python

import sys
import getopt
import os
import re


def usage():
    print "Usage: regexp.py REGEXP FILE points"
    print "Check for pattern REGEXP in the text of file FILE, fail if found."
    print
    print "--regexp=pattern  \t\tThe regular expression to match."
    print "--invert          \t\tSucceed on match, fail on no match."
    sys.exit(2)

def defaults():
    print "Regexp Test"
    print "all"
    print "*.*"
    usage()


try:
    opts, args = getopt.getopt(sys.argv[1:], "", ["help", "defaults",
                                                  "invert", 
                                                  "regexp="])
    
except getopt.GetoptError:
    # print help information and exit:
    usage()
    sys.exit(0)


matchGood = False
pattern = None
for o, a in opts:
    if o == "--help":
        usage()
    if o == "--defaults":
        defaults()

    try:        
        if o == "--invert":
            matchGood = True
        if o == "--regexp":
            pattern = a

    except:
        usage()

if not pattern:
    usage()

try:
    points = float(args.pop())
except:
    print "Bad point value."
    sys.exit(2)

try:
    pattern = re.compile(pattern)
except:
    print "Bad regexp", pattern
    usage()

for fn in args:
    for line in open(fn):
        if pattern.search(line):
            if matchGood:
                print points
            else:
                print 0
                
            print "Pattern matched at:"
            print line
            sys.exit(0)

if matchGood:
    print 0
else:
    print points

print "Pattern not found"
sys.exit(0)


