from memTree import *

from CommentDialog import *

import comments

class UserCommentTree(MemTree):
    """This memtree-based widget just holds the list of user-generated
    comments"""
    
    def __init__(self, parent, agar, *args, **kwds):
        MemTree.__init__(self, "User Comments", *args, **kwds)

        self.parent = parent
        self.a = agar
        self.subtree = self.parent.subTree
        
        self.update()

        # Menu IDs
        self.contextNewID = wx.NewId()
        self.contextEditID = wx.NewId()
        self.contextFindID = wx.NewId()
        wx.EVT_MENU(self, self.contextNewID, self.onNewCommentContext)
        wx.EVT_MENU(self, self.contextEditID, self.onEditCommentContext)


        wx.EVT_TREE_BEGIN_DRAG(self, -1, self.onDrag)
        wx.EVT_RIGHT_UP(self, self.onRightUp)        
        wx.EVT_TREE_ITEM_ACTIVATED(self, -1, self.onDClick)

    def update(self):
        """Draw the tree based on the comments in the userComment object"""
        self.tree.children[:] = []
        
        names = [x for x in self.a.userComments]
        names.sort()

        for name in names:
            self.pathAdd([name])

        MemTree.update(self)

    def onDClick(self, event):
        """Edit when double clicked"""
        self.onEditCommentContext(event)
        
    def onDrag(self, event):
        """Allow drags for drag-and-drop"""
        commentName = str(self.GetItemText(self.GetSelection()))

        if commentName not in self.a.userComments:
            return

        data = wx.CustomDataObject(wx.CustomDataFormat("AgarComment"))
        data.SetData(commentName)

        ds = wx.DropSource(self)
        ds.SetData(data)
        ds.DoDragDrop(True)

    def onRightUp(self, event):
        """Context menu"""
        menu = wx.Menu()
        menu.Append(self.contextNewID, "New Comment")
        menu.Append(self.contextEditID, "Edit Comment")
        menu.Append(self.contextFindID, "Locate Comment")

        self.PopupMenu(menu, event.GetPosition())
        menu.Destroy()

    def onNewCommentContext(self, event):
        """Create a new comment"""
        topNames = [x.name for x in self.a.rubric.getTopLevel()]
        commentNames = self.a.userComments.keys()

        if not topNames:
            dlg = wx.MessageDialog(self, "You must have at least one " +
                                   "rubric item to create comments.",
                                   "Error",
                                   wx.OK|wx.ICON_ERROR)
            dlg.ShowModal()
            dlg.Destroy()
            return

        comment = comments.Comment(self.a.rubric.getTopLevel()[0],
                                   False,
                                   1,
                                   "",
                                   "+")        
                
        dlg = CommentDialog(topNames, commentNames, comment, True, "", self)
        ret = dlg.ShowModal()
        addNew = dlg.addCurBox.GetValue()
        name = dlg.name
        cType = dlg.commentType.GetSelection()
        all = dlg.appliesTo.GetValue()
        testName = dlg.rubricChoice.GetStringSelection()
        
        dlg.Destroy()

        if ret == wx.ID_CANCEL:
            return

        if cType == 0:
            comment.op = "+"
        elif cType == 1:
            comment.op = "*"
        elif cType == 2:
            comment.op = "="
        else:
            raise ValueError

        if all:
            comment.appliesAll = True
        else:
            comment.appliesAll = False

        comment.test = self.a.rubric[testName]

        self.a.userComments[name] = comment
        self.update()

        if addNew:
            try:
                sub = self.a.submissions[self.subtree.getUsername()]
                sub.comments.addComment(comment)
                self.subtree.update()
                self.parent.resultTree.update()
            except AssertionError:
                pass        
                    
    def onEditCommentContext(self, event):
        """Edit an existing comment"""
        selId = self.GetSelection()
        if selId == self.GetRootItem():
            return
        
        selectedName = str(self.GetItemText(selId))
        topNames = [x.name for x in self.a.rubric.getTopLevel()]
        commentNames = self.a.userComments.keys()

        if not topNames:
            dlg = wx.MessageDialog(self, "You must have at least one " +
                                   "rubric item to create comments.",
                                   "Error",
                                   wx.OK|wx.ICON_ERROR)
            dlg.ShowModal()
            dlg.Destroy()
            return        

        comment = self.a.userComments[selectedName]
                
        dlg = CommentDialog(topNames, commentNames, comment, False,
                            selectedName, self)
        ret = dlg.ShowModal()
        name = dlg.name
        testName = dlg.rubricChoice.GetStringSelection()
        
        dlg.Destroy()

        if ret == wx.ID_CANCEL:
            return

        if name != selectedName:
            del self.a.userComments[selectedName]
            self.a.userComments[name] = comment
            
        comment.test = self.a.rubric[testName]

        self.subtree.update()
        self.parent.resultTree.update()

